/*****************************************************************************************************

======================================
By: Jason Ho
Email: cho@mindspring.com
AIM: Bravo585
URL: http://www.geocities.com/dakron15
======================================

This tutorial assumes the knowledge of the Unit Circle, Pythagorean Theorem, and Sines and Cosines.

UNIT CIRCLE
-----------
	 
	 .5*PI		Basically you define 4 points on the unit circle, evenly spaced out (in radians). You
     ___			find the (x,y) coordinates by taking the cosine and sine of the points respectively.
    /   \ 		You then connect the 4 points to create a square. Repeat this same process but shift the
PI |  o  | 0	(x,y) coordinates down and to the right to create another square. Then connect the two
    \___/		squares to form the illusion of a cube. Its sorta like programming the steps to drawing
    				that cube you always drew when you were in 3rd grade.
    1.5*PI				 ___
							|\ _|_
							|_|_| |
			  				 \|__\|
			  				 
*****************************************************************************************************/

#define OPTIMIZE_ROM_CALLS
#define SAVE_SCREEN
#include <tigcclib.h>

short _ti89;
short _ti92plus;

//4 points on the unit circle
float point1;
float point2;
float point3;
float point4;

//Variables to store the x coordinate of the 4 points.
int point1_x;
int point2_x;
int point3_x;
int point4_x;

//Variables to store the y coordinate of the 4 points.
int point1_y;
int point2_y;
int point3_y;
int point4_y;

//Variable to store the virtual screen.
char virtual[3840];

//Determines the shift needed from the first square to the second square.
float shift;

//Changable variables
int size;
float speed;

void _main(void)
{
	//Local variables
	int key = 0;
	
	//Initialize variables
	size = 30;
	speed = 0.1;
	shift = 21.2132;
	
	//Initialize the 4 points on the unit circle, each PI/2 units apart (2 PI in a unit circle)
	point1 = 0.;
	point2 = PI * 0.5;
	point3 = PI;
	point4 = PI * 1.5;
	
	//Sets up the virtual screen.
	PortSet (virtual, 239, 127);
	
	for(;;)	//Loop infinitely
	{
		while(!kbhit())
		{
			//On the unit circle, cosine of a radian gives its x coordinate (range of 0 to 1)
			//times 'size' makes radius of unit circle 'size' times longer; cube gets larger.
			//plus 70 shifts points 70-30 (or 40) pixels to the right.
			point1_x = cos(point1) * size + 70;
			point2_x = cos(point2) * size + 70;
			point3_x = cos(point3) * size + 70;
			point4_x = cos(point4) * size + 70;
			
			//On the unit circle, sin of a radian gives its y coordinate (range of 0 to 1)
			//times 'size' makes radius of unit circle 'size' times longer;cube gets larger.
			//plus 40 shifts points 40-30 (or 10) pixels down.
			point1_y = sin(point1) * size + 40;
			point2_y = sin(point2) * size + 40;
			point3_y = sin(point3) * size + 40;
			point4_y = sin(point4) * size + 40;
			
			//Rotates the points on the unit circle (all points are evenly spaced to look like a cube)
			point1 += speed;
			point2 += speed;
			point3 += speed;
			point4 += speed;
			
			//Draws first square by connecting the points
			DrawLine(point1_x, point1_y, point2_x, point2_y, A_NORMAL);
			DrawLine(point2_x, point2_y, point3_x, point3_y, A_NORMAL);
			DrawLine(point3_x, point3_y, point4_x, point4_y, A_NORMAL);
			DrawLine(point4_x, point4_y, point1_x, point1_y, A_NORMAL);
			
			/*
			 ___
			|  _|_	<-- original square
			|_|_| |
			  |___|	<-- shifted square
			
			*/
			
			shift = sqrt( pow(size,2) + pow(size,2) )/2;
			
			//Draws the second square; this is a shifting of the first square
			//Shift is exactly half of a side of the square. The center of the unit circle to a point
			//on the unit circle is half the diagonal of the square...so do the math to find the shift ;)
			DrawLine(point1_x+shift, point1_y+shift, point2_x+shift, point2_y+shift, A_NORMAL);
			DrawLine(point2_x+shift, point2_y+shift, point3_x+shift, point3_y+shift, A_NORMAL);
			DrawLine(point3_x+shift, point3_y+shift, point4_x+shift, point4_y+shift, A_NORMAL);
			DrawLine(point4_x+shift, point4_y+shift, point1_x+shift, point1_y+shift, A_NORMAL);
			
			//Links the two squares together to form a cube.
			DrawLine(point1_x, point1_y, point1_x+shift, point1_y+shift, A_NORMAL);
			DrawLine(point2_x, point2_y, point2_x+shift, point2_y+shift, A_NORMAL);
			DrawLine(point3_x, point3_y, point3_x+shift, point3_y+shift, A_NORMAL);
			DrawLine(point4_x, point4_y, point4_x+shift, point4_y+shift, A_NORMAL);
			
			//Draws the virtual screen to the actual screen.
			memcpy (LCD_MEM, virtual, LCD_SIZE);
			
			//Clear the screen.
			ClrScr();
		}
		//Store pressed button to key
		key = ngetchx();
		
		switch(key)
		{
			case 43:		//Press + to increase speed of rotation
				speed += 0.01;
				break;
			case 45:		//Press - to decrease speed of rotation
				speed -= 0.01;
				break;
			case 268:	//Press F1 to increase size of cube
				size += 1;
				if(size > 40) size = 40;
				break;
			case 269:	//Press F2 to decrease size of cube
				size -= 1;
				if(size < -40) size = -40;
				break;
			case 13:		//Press 13 to change the direction of rotation of the cube.
				speed *= -1;
				break;
			case 264:	//Press ESC to escape
				PortRestore();
				return;
				break;
		}
	}
}
